@@ -1,18 +1,25 @@
+import re
+
+from aioesphomeapi.model import BluetoothGATTError
+
 from .api_pb2 import (  # type: ignore
     BinarySensorStateResponse,
     BluetoothConnectionsFreeResponse,
     BluetoothDeviceConnectionResponse,
     BluetoothDeviceRequest,
+    BluetoothGATTErrorResponse,
     BluetoothGATTGetServicesDoneResponse,
     BluetoothGATTGetServicesRequest,
     BluetoothGATTGetServicesResponse,
     BluetoothGATTNotifyDataResponse,
     BluetoothGATTNotifyRequest,
+    BluetoothGATTNotifyResponse,
     BluetoothGATTReadDescriptorRequest,
     BluetoothGATTReadRequest,
     BluetoothGATTReadResponse,
     BluetoothGATTWriteDescriptorRequest,
     BluetoothGATTWriteRequest,
+    BluetoothGATTWriteResponse,
     BluetoothLEAdvertisementResponse,
     ButtonCommandRequest,
     CameraImageRequest,
@@ -82,6 +89,55 @@ from .api_pb2 import (  # type: ignore
     TextSensorStateResponse,
 )
 
+TWO_CHAR = re.compile(r".{2}")
+
+# Taken from esp_gatt_status_t in esp_gatt_defs.h
+ESPHOME_GATT_ERRORS = {
+    -1: "Not connected",  # Custom ESPHome error
+    1: "Invalid handle",
+    2: "Read not permitted",
+    3: "Write not permitted",
+    4: "Invalid PDU",
+    5: "Insufficient authentication",
+    6: "Request not supported",
+    7: "Invalid offset",
+    8: "Insufficient authorization",
+    9: "Prepare queue full",
+    10: "Attribute not found",
+    11: "Attribute not long",
+    12: "Insufficient key size",
+    13: "Invalid attribute length",
+    14: "Unlikely error",
+    15: "Insufficient encryption",
+    16: "Unsupported group type",
+    17: "Insufficient resources",
+    128: "Application error",
+    129: "Internal error",
+    130: "Wrong state",
+    131: "Database full",
+    132: "Busy",
+    133: "Error",
+    134: "Command started",
+    135: "Illegal parameter",
+    136: "Pending",
+    137: "Auth fail",
+    138: "More",
+    139: "Invalid configuration",
+    140: "Service started",
+    141: "Encrypted no mitm",
+    142: "Not encrypted",
+    143: "Congested",
+    144: "Duplicate registration",
+    145: "Already open",
+    146: "Cancel",
+    224: "Stack RSP",
+    225: "App RSP",
+    239: "Unknown error",
+    253: "CCC config error",
+    254: "Procedure already in progress",
+    255: "Out of range",
+}
+
 
 class APIConnectionError(Exception):
     pass
@@ -139,6 +195,28 @@ class ReadFailedAPIError(APIConnectionError):
     pass
 
 
+def to_human_readable_address(address: int) -> str:
+    """Convert a MAC address to a human readable format."""
+    return ":".join(TWO_CHAR.findall(f"{address:012X}"))
+
+
+def to_human_readable_gatt_error(error: int) -> str:
+    """Convert a GATT error to a human readable format."""
+    return ESPHOME_GATT_ERRORS.get(error, "Unknown error")
+
+
+class BluetoothGATTAPIError(APIConnectionError):
+    def __init__(self, error: BluetoothGATTError) -> None:
+        super().__init__(
+            f"Bluetooth GATT Error "
+            f"address={to_human_readable_address(error.address)} "
+            f"handle={error.handle} "
+            f"error={error.error} "
+            f"description={to_human_readable_gatt_error(error.error)}"
+        )
+        self.error = error
+
+
 MESSAGE_TYPE_TO_PROTO = {
     1: HelloRequest,
     2: HelloResponse,
@@ -221,4 +299,7 @@ MESSAGE_TYPE_TO_PROTO = {
     79: BluetoothGATTNotifyDataResponse,
     80: SubscribeBluetoothConnectionsFreeRequest,
     81: BluetoothConnectionsFreeResponse,
+    82: BluetoothGATTErrorResponse,
+    83: BluetoothGATTWriteResponse,
+    84: BluetoothGATTNotifyResponse,
 }
